@@ -43,6 +43,14 @@ def add_path(path1, path2):
     return os.path.join(path1, path2)
 
 
+def try_to_get_absolutely_path(path):
+    abs_path = os.path.abspath(path)
+    if os.path.isabs(abs_path) or os.path.exists(abs_path):
+        return abs_path
+    else:
+        return path
+
+
 def make_sure_source_is_absolute_path(path):
     path_list = []
     if isinstance(path, str):
@@ -88,12 +96,15 @@ def make_sure_target_does_not_exist(path):
             else:
                 r = os.remove(p)
                 if r is False:
-                    print(f"I can't remove target '{p}' for you, please check your permission")
+                    print(
+                        f"I can't remove target '{p}' for you, please check your permission")
                     exit()
 
 
 def convert_video_to_wav(source_video_path, target_wav_path):
+    source_video_path = try_to_get_absolutely_path(source_video_path)
     make_sure_source_is_absolute_path(source_video_path)
+    target_wav_path = try_to_get_absolutely_path(target_wav_path)
     make_sure_target_is_absolute_path(target_wav_path)
 
     make_sure_target_does_not_exist(target_wav_path)
@@ -106,6 +117,7 @@ def convert_video_to_wav(source_video_path, target_wav_path):
 
 
 def get_wav_infomation(wav_path):
+    wav_path = try_to_get_absolutely_path(wav_path)
     make_sure_source_is_absolute_path(wav_path)
 
     y, sr = librosa.load(wav_path, sr=None)
@@ -118,13 +130,9 @@ class Video():
     This is a powerful video module
 
     It's based on:
-
     1. ubuntu core
-
     2. librosa
-
     3. ffmpeg
-
     4. moviepy
     """
 
@@ -356,7 +364,7 @@ class Video():
 
         done()
 
-    def link_videos(self, source_video_path_list, target_video_path, method=1, preset="ultrafast"):
+    def link_videos(self, source_video_path_list, target_video_path, method=2, preset="ultrafast"):
         """
         concatenate videos one by one
 
@@ -371,7 +379,10 @@ class Video():
         method: int
             1 to use ffmpeg(low quality), 2 to use moviepy(high quality)
         """
+        source_video_path_list = [try_to_get_absolutely_path(
+            f) for f in source_video_path_list]
         make_sure_source_is_absolute_path(source_video_path_list)
+        target_video_path = try_to_get_absolutely_path(target_video_path)
         make_sure_target_is_absolute_path(target_video_path)
 
         if method == 1:
@@ -391,9 +402,11 @@ class Video():
         elif method == 2:
             # for the stupid moviepy library, it will case memory leak if you give it too much videos
             print(source_video_path_list)
-            clip_list = [VideoFileClip(clip) for clip in source_video_path_list]
+            clip_list = [VideoFileClip(clip)
+                         for clip in source_video_path_list]
             final_clip = concatenate_videoclips(clip_list)
-            final_clip.write_videofile(target_video_path, threads=self._cpu_core_numbers, preset=preset)
+            final_clip.write_videofile(
+                target_video_path, threads=self._cpu_core_numbers, preset=preset)
 
             for clip in clip_list:
                 clip.close()
@@ -438,7 +451,9 @@ class Video():
         """
         degree = str(degree)
 
+        source_video_path = try_to_get_absolutely_path(source_video_path)
         make_sure_source_is_absolute_path(source_video_path)
+        target_video_path = try_to_get_absolutely_path(target_video_path)
         make_sure_target_is_absolute_path(target_video_path)
 
         if not noise_capture_length:
@@ -497,6 +512,9 @@ class Video():
         voice_only: bool
             if true, it only returns the path of silence removed mp3 file
         """
+
+        source_video_path = try_to_get_absolutely_path(source_video_path)
+        target_video_path = try_to_get_absolutely_path(target_video_path)
         make_sure_source_is_absolute_path(source_video_path)
         make_sure_source_is_absolute_path(target_video_path)
 
@@ -504,9 +522,9 @@ class Video():
 
         working_dir = get_directory_name(target_video_path)
         audio_path = convert_video_to_wav(source_video_path, add_path(
-            working_dir, 'audio_for_remove_silence_parts_from_video.wav'))
+            working_dir, disk.get_hash_of_a_path(source_video_path)+'audio_for_remove_silence_parts_from_video.wav'))
         temp_video_path = add_path(
-            working_dir, 'temp_for_remove_silence_parts_from_video.mp4')
+            working_dir, disk.get_hash_of_a_path(source_video_path)+'temp_for_remove_silence_parts_from_video.mp4')
 
         if minimum_interval_time_in_seconds is None:
             parts = self._get_voice_parts(audio_path, top_db)
@@ -520,8 +538,10 @@ class Video():
         length = len(parts)
         for index, part in enumerate(parts):
             try:
-                time_duration = (datetime.datetime.strptime(part[1], '%H:%M:%S.%f') - datetime.datetime.strptime(part[0], '%H:%M:%S.%f')).seconds
-                print(str(int(index/length*100))+"%,", "-".join([p.split(".")[0] for p in part]) + ",", "cut " + str(time_duration) + " seconds")
+                time_duration = (datetime.datetime.strptime(
+                    part[1], '%H:%M:%S.%f') - datetime.datetime.strptime(part[0], '%H:%M:%S.%f')).seconds
+                print(str(int(index/length*100))+"%,", "-".join(
+                    [p.split(".")[0] for p in part]) + ",", "cut " + str(time_duration) + " seconds")
             except Exception as e:
                 print(e)
             clip_list.append(parent_clip.subclip(part[0], part[1]))
@@ -529,14 +549,16 @@ class Video():
         concat_clip = concatenate_videoclips(clip_list)
 
         if not voice_only:
-            concat_clip.write_videofile(target_video_path, threads=self._cpu_core_numbers)
+            concat_clip.write_videofile(
+                target_video_path, threads=self._cpu_core_numbers)
             concat_clip.close()
             del concat_clip
             make_sure_target_does_not_exist(audio_path)
             make_sure_target_does_not_exist(temp_video_path)
         else:
             if len(target_video_path.split(".")) >= 2:
-                target_audio_path = ".".join(target_video_path.split(".")[:-1]) + ".mp3"
+                target_audio_path = ".".join(
+                    target_video_path.split(".")[:-1]) + ".mp3"
             else:
                 target_audio_path = target_video_path + ".mp3"
             make_sure_target_does_not_exist(target_audio_path)
@@ -562,6 +584,8 @@ class Video():
         minimum_interval_time_in_seconds: float
             longer than this value, we will take it as silence and remove it
         """
+        source_video_path = try_to_get_absolutely_path(source_video_path)
+        target_video_path = try_to_get_absolutely_path(target_video_path)
         source_video_path = os.path.abspath(source_video_path)
         target_video_path = os.path.abspath(target_video_path)
 
@@ -612,25 +636,33 @@ class Video():
         speed: float
             how quick you want the video to be
         """
+        source_video_path = try_to_get_absolutely_path(source_video_path)
+        target_video_path = try_to_get_absolutely_path(target_video_path)
         make_sure_source_is_absolute_path(source_video_path)
         make_sure_target_is_absolute_path(target_video_path)
         make_sure_target_does_not_exist(target_video_path)
 
         speed = float(speed)
-
         video_speed = str(float(1/speed))[:6]
 
-        parts = math.ceil(speed/2)
-        value_of_each_part = str(speed ** (1/parts))[:6]
-        audio_speed = ",".join([f"atempo={value_of_each_part}" for i in range(parts)])
+        if speed <= 4:
+            parts = math.ceil(speed/2)
+            value_of_each_part = str(speed ** (1/parts))[:6]
+            audio_speed = ",".join(
+                [f"atempo={value_of_each_part}" for i in range(parts)])
 
-        t.run(f"""
-            ffmpeg -i "{source_video_path}" -filter_complex "[0:v]setpts={video_speed}*PTS[v];[0:a]{audio_speed}[a]" -map "[v]" -map "[a]" "{target_video_path}"
-        """)
+            t.run(f"""
+                ffmpeg -i "{source_video_path}" -filter_complex "[0:v]setpts={video_speed}*PTS[v];[0:a]{audio_speed}[a]" -map "[v]" -map "[a]" "{target_video_path}"
+            """)
 
-        done()
+            done()
+        else:
+            self._speedup_video_with_moviepy(
+                source_video_path, target_video_path, speed=speed)
 
     def _speedup_video_with_moviepy(self, source_video_path, target_video_path, speed=4):
+        source_video_path = try_to_get_absolutely_path(source_video_path)
+        target_video_path = try_to_get_absolutely_path(target_video_path)
         make_sure_source_is_absolute_path(source_video_path)
         make_sure_target_is_absolute_path(target_video_path)
         make_sure_target_does_not_exist(target_video_path)
@@ -654,6 +686,8 @@ class Video():
         speed: float
             how quick you want the silence parts to be
         """
+        source_video_path = try_to_get_absolutely_path(source_video_path)
+        target_video_path = try_to_get_absolutely_path(target_video_path)
         make_sure_source_is_absolute_path(source_video_path)
         make_sure_source_is_absolute_path(target_video_path)
 
@@ -661,7 +695,7 @@ class Video():
 
         working_dir = get_directory_name(target_video_path)
         audio_path = convert_video_to_wav(source_video_path, add_path(
-            working_dir, 'audio_for_speedup_silence_parts_in_video.wav'))
+            working_dir, disk.get_hash_of_a_path(source_video_path)+'audio_for_speedup_silence_parts_in_video.wav'))
 
         voice_and_silence_parts = self._get_voice_and_silence_parts(
             audio_path, top_db)
@@ -674,8 +708,10 @@ class Video():
             if len(part[1][0].split(".")) < 2:
                 part[1][0] += ".000000"
             try:
-                time_duration = (datetime.datetime.strptime(part[1][1], '%H:%M:%S.%f') - datetime.datetime.strptime(part[1][0], '%H:%M:%S.%f')).seconds
-                print(str(int(index/length*100))+"%,", "-".join([p.split(".")[0] for p in part[1]]) + ",", "cut " + str(time_duration) + " seconds")
+                time_duration = (datetime.datetime.strptime(
+                    part[1][1], '%H:%M:%S.%f') - datetime.datetime.strptime(part[1][0], '%H:%M:%S.%f')).seconds
+                print(str(int(index/length*100))+"%,", "-".join([p.split(".")[
+                      0] for p in part[1]]) + ",", "cut " + str(time_duration) + " seconds")
             except Exception as e:
                 print(e)
             if part[0] == 1:  # voice
@@ -689,7 +725,8 @@ class Video():
 
         concat_clip = concatenate_videoclips(clip_list)
 
-        concat_clip.write_videofile(target_video_path, threads=self._cpu_core_numbers)
+        concat_clip.write_videofile(
+            target_video_path, threads=self._cpu_core_numbers)
         concat_clip.close()
         del concat_clip
 
@@ -704,6 +741,9 @@ class Video():
         picture: string
             the path of that picture
         """
+        source_video_path = try_to_get_absolutely_path(source_video_path)
+        target_video_path = try_to_get_absolutely_path(target_video_path)
+        picture = try_to_get_absolutely_path(picture)
         make_sure_source_is_absolute_path(source_video_path)
         make_sure_target_is_absolute_path(target_video_path)
         make_sure_target_is_absolute_path(picture)
@@ -716,6 +756,8 @@ class Video():
         done()
 
     def get_mp3_from_video(self, source_video_path, target_video_path):
+        source_video_path = try_to_get_absolutely_path(source_video_path)
+        target_video_path = try_to_get_absolutely_path(target_video_path)
         make_sure_source_is_absolute_path(source_video_path)
         make_sure_target_is_absolute_path(target_video_path)
         make_sure_target_does_not_exist(target_video_path)
@@ -730,6 +772,8 @@ class Video():
         done()
 
     def delay_audio_in_video(self, source_video_path, target_video_path, delay):
+        source_video_path = try_to_get_absolutely_path(source_video_path)
+        target_video_path = try_to_get_absolutely_path(target_video_path)
         make_sure_source_is_absolute_path(source_video_path)
         make_sure_target_is_absolute_path(target_video_path)
         make_sure_target_does_not_exist(target_video_path)
@@ -752,6 +796,8 @@ class Video():
         times: float
             2 times the source_video volume or 3 times the source_video volume
         """
+        source_video_path = try_to_get_absolutely_path(source_video_path)
+        target_video_path = try_to_get_absolutely_path(target_video_path)
         make_sure_source_is_absolute_path(source_video_path)
         make_sure_target_is_absolute_path(target_video_path)
         make_sure_target_does_not_exist(target_video_path)
@@ -765,7 +811,8 @@ class Video():
 
         done()
 
-    def compress_videos_in_a_folder(self, source_folder):
+    def compress_videos_in_a_folder(self, source_folder, fps: int = 29, resolution: Tuple[int, int] = None, preset: str = 'placebo'):
+        source_folder = try_to_get_absolutely_path(source_folder)
         make_sure_source_is_absolute_path(source_folder)
 
         working_dir = get_directory_name(source_folder)
@@ -793,17 +840,22 @@ class Video():
             target_video_path = add_path(new_folder, basename)
             make_sure_target_does_not_exist(target_video_path)
 
-            size, unit = convert_bytes(os.path.getsize(file))
-            if unit == "GB":
-                if size > 2:
-                    t.run(f"""
-                        # ffmpeg -i "{file}" "{target_video_path}"
-                        ffmpeg -i "{file}" -c copy -c:v libx264 -vf scale=-2:720 "{target_video_path}"
-                    """)
+            #size, unit = convert_bytes(os.path.getsize(file))
+            # if unit == "GB":
+            #    if size > 2:
+            if fps and resolution and preset:
+                t.run(f"""
+                    ffmpeg -i "{file}" -c copy -c:v libx264 -vf scale={resolution[0]}:{resolution[1]} -r {fps} -preset {preset} "{target_video_path}"
+                """)
+            else:
+                t.run(f"""
+                    ffmpeg -i "{file}" -c copy -c:v libx264 -vf scale=-2:720 "{target_video_path}"
+                """)
 
         done()
 
     def format_videos_in_a_folder(self, source_folder):
+        source_folder = try_to_get_absolutely_path(source_folder)
         make_sure_source_is_absolute_path(source_folder)
 
         working_dir = get_directory_name(source_folder)
@@ -833,7 +885,7 @@ class DeepVideo():
 
     It's based on:
     1. ubuntu core
-    2. librosa
+    2. vosk
     3. ffmpeg
     4. moviepy
     """
@@ -843,14 +895,16 @@ class DeepVideo():
 
         self._cpu_core_numbers = multiprocessing.cpu_count()
 
-        self.config_folder = Path("~/.auto_everything").expanduser() / Path("video")
+        self.config_folder = Path(
+            "~/.auto_everything").expanduser() / Path("video")
         if not os.path.exists(self.config_folder):
             t.run_command(f"mkdir -p {self.config_folder}")
 
         # download
         zip_file = self.config_folder / Path("vosk-en.zip")
         if not disk.exists(zip_file):
-            success = network.download("http://alphacephei.com/kaldi/models/vosk-model-small-en-us-0.3.zip", zip_file, "30MB")
+            success = network.download(
+                "http://alphacephei.com/kaldi/models/vosk-model-small-en-us-0.3.zip", zip_file, "30MB")
             if success:
                 print("download successfully")
             else:
@@ -910,7 +964,8 @@ class DeepVideo():
     def __get_data_from_video(self, path: str, minimum_interval_time_in_seconds: float = 0.0, video_length: float = None):
         from vosk import Model, KaldiRecognizer, SetLogLevel
 
-        assert os.path.exists(path), f"source video file {path} does not exist!"
+        assert os.path.exists(
+            path), f"source video file {path} does not exist!"
 
         SetLogLevel(0)
         sample_rate = 16000
@@ -957,19 +1012,23 @@ class DeepVideo():
         """
         make_sure_target_does_not_exist(target_video_path)
 
-        video_files = [video for video in disk.get_files(source_folder, recursive=False)]
+        video_files = [video for video in disk.get_files(
+            source_folder, recursive=False)]
         video_files = disk.sort_files_by_time(video_files)
         if resolution != None:
             resolution = (resolution[1], resolution[0])
-        parent_clips = [VideoFileClip(video, target_resolution=resolution) for video in video_files]
+        parent_clips = [VideoFileClip(
+            video, target_resolution=resolution) for video in video_files]
         length = len(video_files)
         remain_clips = []
         for index, video_file in enumerate(video_files):
             print("Working on ", str(int(index/length*100)) + " %...")
-            parts = self.__get_data_from_video(video_file, minimum_interval_time_in_seconds, parent_clips[index].duration)
+            parts = self.__get_data_from_video(
+                video_file, minimum_interval_time_in_seconds, parent_clips[index].duration)
             for part in parts:
                 if part[2] == 'voice':
-                    remain_clips.append(parent_clips[index].subclip(part[0], part[1]))
+                    remain_clips.append(
+                        parent_clips[index].subclip(part[0], part[1]))
 
         concat_clip = concatenate_videoclips(remain_clips)
         concat_clip.write_videofile(target_video_path, fps=fps, preset=preset)
@@ -999,19 +1058,23 @@ class DeepVideo():
         """
         make_sure_target_does_not_exist(target_video_path)
 
-        video_files = [video for video in disk.get_files(source_folder, recursive=False)]
+        video_files = [video for video in disk.get_files(
+            source_folder, recursive=False)]
         video_files = disk.sort_files_by_time(video_files)
         if resolution != None:
             resolution = (resolution[1], resolution[0])
-        parent_clips = [VideoFileClip(video, target_resolution=resolution) for video in video_files]
+        parent_clips = [VideoFileClip(
+            video, target_resolution=resolution) for video in video_files]
         length = len(video_files)
         remain_clips = []
         for index, video_file in enumerate(video_files):
             print("Working on ", str(int(index/length*100)) + " %...")
-            parts = self.__get_data_from_video(video_file, minimum_interval_time_in_seconds, parent_clips[index].duration)
+            parts = self.__get_data_from_video(
+                video_file, minimum_interval_time_in_seconds, parent_clips[index].duration)
             for part in parts:
                 if part[2] == 'voice':
-                    remain_clips.append(parent_clips[index].subclip(part[0], part[1]))
+                    remain_clips.append(
+                        parent_clips[index].subclip(part[0], part[1]))
                 elif part[2] == 'silence':
                     remain_clips.append(
                         parent_clips[index].subclip(part[0], part[1]).without_audio().fx(
@@ -1039,20 +1102,23 @@ class DeepVideo():
         make_sure_target_does_not_exist(target_video_path)
 
         parent_clip = VideoFileClip(source_video_path)
-        parts = self.__get_data_from_video(source_video_path, minimum_interval_time_in_seconds, parent_clip.duration)
+        parts = self.__get_data_from_video(
+            source_video_path, minimum_interval_time_in_seconds, parent_clip.duration)
         clip_list = []
         length = len(parts)
         for index, part in enumerate(parts):
             if part[2] == 'voice':
                 try:
                     time_duration = part[1] - part[0]
-                    print(str(int(index/length*100))+"%,", "remain " + str(int(time_duration)) + " seconds")
+                    print(str(int(index/length*100))+"%,", "remain " +
+                          str(int(time_duration)) + " seconds")
                 except Exception as e:
                     print(e)
                 clip_list.append(parent_clip.subclip(part[0], part[1]))
 
         concat_clip = concatenate_videoclips(clip_list)
-        concat_clip.write_videofile(target_video_path, threads=self._cpu_core_numbers)
+        concat_clip.write_videofile(
+            target_video_path, threads=self._cpu_core_numbers)
         concat_clip.close()
         parent_clip.close()
 
@@ -1071,8 +1137,10 @@ class DeepVideo():
         """
         make_sure_target_does_not_exist(target_video_path)
 
-        parent_clip = VideoFileClip(source_video_path, target_resolution=(resolution[1], resolution[0]))
-        parts = self.__get_data_from_video(source_video_path, minimum_interval_time_in_seconds, parent_clip.duration)
+        parent_clip = VideoFileClip(
+            source_video_path, target_resolution=(resolution[1], resolution[0]))
+        parts = self.__get_data_from_video(
+            source_video_path, minimum_interval_time_in_seconds, parent_clip.duration)
         clip_list = []
         length = len(parts)
         for index, part in enumerate(parts):
@@ -1081,7 +1149,8 @@ class DeepVideo():
             elif part[2] == 'silence':
                 try:
                     time_duration = part[1] - part[0]
-                    print(str(int(index/length*100))+"%,", "speed up " + str(int(time_duration)) + " seconds")
+                    print(str(int(index/length*100))+"%,", "speed up " +
+                          str(int(time_duration)) + " seconds")
                 except Exception as e:
                     print(e)
                 clip_list.append(
@@ -1091,7 +1160,8 @@ class DeepVideo():
                 )
 
         concat_clip = concatenate_videoclips(clip_list)
-        concat_clip.write_videofile(target_video_path, threads=self._cpu_core_numbers, fps=fps, preset=preset)
+        concat_clip.write_videofile(
+            target_video_path, threads=self._cpu_core_numbers, fps=fps, preset=preset)
         concat_clip.close()
         parent_clip.close()
 
@@ -1100,4 +1170,5 @@ class DeepVideo():
 
 if __name__ == "__main__":
     video = DeepVideo()
-    video.speedup_silence_parts_in_video("/home/yingshaoxo/demo.mp4", "/home/yingshaoxo/ok.mp4", 10)
+    video.speedup_silence_parts_in_video(
+        "/home/yingshaoxo/demo.mp4", "/home/yingshaoxo/ok.mp4", 10)
